from xml.dom.minidom import parse
__all__ = ['parsexml']


is_text_node = lambda x: x.nodeType == x.TEXT_NODE


def extractchildtext(node):
    """given a node, extract all the text from it's child nodes"""
    return "".join(n.nodeValue for n in filter(is_text_node, node.childNodes))


def getnodeattribute(node, attrname):
    """given an xmlnode, retrieve its attribute named attrname else raise a
    KeyError exception"""
    if node.hasAttributes:
        attributes = node.attributes
        for idx in xrange(attributes.length):
            attr = attributes.item(idx)
            if attr.name == attrname:
                return attr.value
    raise KeyError("Uknown attr key %s" % attrname)


def parsedict(node):
    """parse a node that is a dict, for each child node it's nodeName
    is they key and it's childNodes are it's value"""
    d = {}
    for child in node.childNodes:
        d[child.nodeName] = parsenode(child)
    return d


def parselist(node):
    """parse all childNodes adding them to a list"""
    return [parsenode(child) for child in node.childNodes]


def parseint(node):
    """return an int from the nodes child text nodes"""
    return int(extractchildtext(node))


def parsefloat(node):
    """return a float from the child text nodes"""
    return float(extractchildtext(node))


def parsestring(node):
    """return a string that is stripped of pre/post whitespace from the
    child text nodes"""
    return extractchildtext(node).strip()


def parsevector(node):
    """parse a comma seperated list of floats from the nodes
    child text nodes"""
    s = extractchildtext(node)
    return map(float, s.split(','))


def parsebool(node):
    """parse the child text nodes as an int and then return False for
    0 otherwise True"""
    v = parseint(node)
    return v != 0


# map between node types (as defined by their type parameter)
# and a function to parse it and it's child nodes
parsefunctions = {}
parsefunctions['list'] = parselist
parsefunctions['dict'] = parsedict
parsefunctions['int'] = parseint
parsefunctions['float'] = parsefloat
parsefunctions['string'] = parsestring
parsefunctions['vector2'] = parsevector
parsefunctions['vector3'] = parsevector
parsefunctions['path'] = parsestring
parsefunctions['relpath'] = parsestring
parsefunctions['quarternion'] = parsevector
parsefunctions['date'] = parsestring
parsefunctions['bool'] = parsebool


def parsenode(node):
    """recursive functon to parse a node tree based on the type parameter
    in the node return a python type"""
    try:
        t = getnodeattribute(node, "type")
    except KeyError:
        raise Exception("Node doesn't have type paramter: %s" % node.nodeName)

    if t in parsefunctions:
        return parsefunctions[t](node)
    raise Exception("Uknown node type: %s", t)


def parsexml(path):
    xml = parse(path)
    blend = parsenode(xml.documentElement)

    return blend

